@@ -4,7 +4,6 @@
 #include <glib.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
 #include <gtk/gtkwidget.h>
-#include <gtk/gtktooltips.h>
 #include <panel-applet.h>
 
 G_BEGIN_DECLS
@@ -38,7 +37,6 @@ struct _LoadGraph {
     GdkPixmap *pixmap;
     GdkGC *gc;
     int timer_index;
-    GtkTooltips *tooltips;
 
     gint show_frame;
 
@@ -79,18 +77,18 @@ struct _MultiloadApplet
 #include "linux-proc.h"
 
 /* show properties dialog */
-void
+G_GNUC_INTERNAL void
 multiload_properties_cb (BonoboUIComponent *uic,
 			 MultiloadApplet   *ma,	
-			 const char        *name) G_GNUC_INTERNAL;
+			 const char        *name);
 
 /* remove the old graphs and rebuild them */
-void
-multiload_applet_refresh(MultiloadApplet *ma) G_GNUC_INTERNAL;
+G_GNUC_INTERNAL void
+multiload_applet_refresh(MultiloadApplet *ma);
 
 /* update the tooltip to the graph's current "used" percentage */
-void
-multiload_applet_tooltip_update(LoadGraph *g) G_GNUC_INTERNAL;
+G_GNUC_INTERNAL void
+multiload_applet_tooltip_update(LoadGraph *g);
 
 G_END_DECLS
 
